package com.zhongzheng.modules.exam.mapper;

import com.zhongzheng.modules.exam.bo.ExamApplyQueryBo;
import com.zhongzheng.modules.exam.bo.ExamNumberGoodsQueryBo;
import com.zhongzheng.modules.exam.domain.ExamApply;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.zhongzheng.modules.exam.vo.ExamApplySiteTimeVo;
import com.zhongzheng.modules.exam.vo.ExamApplySiteVo;
import com.zhongzheng.modules.exam.vo.ExamApplyVo;
import com.zhongzheng.modules.exam.vo.ExamNumberGoodsVo;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 考试安排Mapper接口
 *
 * @author ruoyi
 * @date 2021-12-07
 */
public interface ExamApplyMapper extends BaseMapper<ExamApply> {

    List<ExamNumberGoodsVo> listGoods(ExamNumberGoodsQueryBo bo);

    List<Long> countGoods(Long applyId);

    List<ExamNumberGoodsVo> examNumberGoodsVos(Long applyId);

    List<ExamApplySiteVo> addressExam(@Param("applyId") Long applyId,@Param("status") Long i);

    List<ExamApplySiteTimeVo> addressTimeExam(Long id);

    ExamApplyVo selectByApply(Long applyId);

    List<ExamApplyVo> queryExam(ExamApplyQueryBo bo);
}
